%
% GENERIC LINK BUDGET CALCULATIONS TEMPLATE
%
% This template implements the basic link budget in MATLAB and can be
% further extended in order to extract parameters in the link budget and to
% tweak parameters in the link budget. All the link budget parameters will
% be available in this workspace for manipulation. Without further
% manipulations the two lines of code will compute the link budget as is
% the excel sheet provided by Asbjrn Dahl .
%
% Example of use 1: Extract SNR link margin
% It is found in exec_link_budget.m that the SNR link margin is stored in
% the variable LM_snr. This will be stored to workspace and can simply by
% read by executing "LM_snr" on the end o f the script .
%
% Example of use 2: Compare different budgets
% Add a loop around the whole script and iterate through all budgets of
% interest. The link budget variable names will be over written in each
% iteration so store the variables of interest into an array that will
% contain them after the loop has run. NB: import_link_vars( ) can also
% take the budget number as argument instead of two text strings
% describing which budget it is. It plays nicer with loops.
%
% Example of use 3: Plot link budget variable versus elevation
% Create a vector of different elevation angles. Import link budget
% variables with import_link_vars() as usual but over write the
% elevation-dependent variables with vectors corresponding to the
% different elevations afterwards. There are functions created for this.
% Run exec_link_budget as usual and plot elevation versus the variable of
% interest, i.e. link margin.
%
% Example of use 4: Plot link budget varibles versus time
% Assume circular orbit. Compute elevation angles versus time and follow
% example 3 but plot versus time instead of elevation. There are
% function screated for this .
%
% Example of use 5: Synthesize ideal antenna gain (advanced)
% Load variables and replace the satellite antenna with an isotropic one
% and compute link margin for a sweep of elevation angles. De fine the
% ideal link margin to be constant (at least above some cut-off elevation
% angle) and take the difference between ideal and isotropic link margin
% to be the preliminary antenna gain versus elevation. Convert to gain
% versus theta. Antenna gain will be unphysical as the ideal link
% margin is arbitrarily chosen. Normalize such that average gain is 1.
% Recompute link budget with exec_link_budget with new parameters to get
% actual link margin
%
% Overview of different budgets:
%
% 1-UHF Downlink
% 2-UHF Uplink
% 3-VHF Downlink
% 4-VHF Upl ink
% 5-Beacon Downlink
% 6-Beacon Upl ink
%
% import_vars() can take either budget numer or strings as arguments
%
% LOAD NECESSARY VARIABLES WITH DEFAULT VALUES
%
% See link_data.txt for details about the variables .
% All variables are converted to SI-units .
%
%clear all
%close all
%clc
%============================= NOTES ====================================
% Threshold values for BFSK and BPSK were added to link_data.txt file to 
% figure out the right Link Margin. In addition, rates of 9600 and 1800  
% also have been included. Lines codes to download Eb/No for both cases
% have been added, to calculate Link Margin calculation and generate graphs
% as well. link_budget.m is called on bfsk.m and bpsk.m since Eb/No is 
% required to figure out Error Probability.
%========================================================================

import_vars('link_data.txt',3) ;       % Select frequency%
evalin('caller','f=f*1e6;') ;          % MHz > Hz
evalin('caller','A_h = A_h*1e3 ; ') ;  % km  > m
evalin('caller','B=B*1e3; ' ) ;        % kHz > Hz

h=[400 600 800]*1e3 ;                  % Heights
t=0:1000 ;                             % Time after rise [s]
el=0:0.09:90;

h0=waitbar(0,'Executing Link Budget');

for i=1:3
waitbar(i/3);
adcs_acc = 2 ;                        % Accuracy of ADCS [degree] 
              
los=los_vs_elevation(el,h(i)) ;       % Slant range [m]
theta=offset_angle_vs_elevation(el,h(i)); % Antenna off set angle [degree]
theta=theta+adcs_acc ;                % Add ADCS uncertainty [degree]

% Replace Link Budget variables
L_plf=3 ;                             
L_tx_pnt=0 ;                          % Included in G_tx

  if dir==1
   G_tx=pattern_dipole(theta,true,true); % Diagram(normalized)
   G_tx=G_tx+2.15;                       % Denormalize
  else 
   G_tx=G_tx;
  end 

L_fspl=fspl_vs_los(los,f) ;           % FSPL [dB]

atmloss_vs_elevation(el,f) ;         
L_atm=atmloss_vs_elevation(el,f) ;    % Atmosferic losses [dB]

L_ion=ionloss_vs_elevation(el,f) ;                             % Ionospheric losses [dB]

exec_link_budget ;                    % Run link budget 

% Load some useful results from exec_link_budget for both rates.
E_ebno_96(1,i,:)=EbNo_96 ;
E_ebno_18(1,i,:)=EbNo_18;

E_LM_ebno_96_fsk(1,i,:)=LM_ebno96_fsk ;
E_LM_ebno_18_fsk(1,i,:)=LM_ebno18_fsk ;

E_LM_ebno_96_psk(1,i,:)=LM_ebno96_psk ;
E_LM_ebno_18_psk(1,i,:)=LM_ebno18_psk ;

end
close(h0);


% Plot Eb/No vs. Elevation and Eb/No Threshold BER=10e-5
figure
hold on
title('Eb/No received with Rate=9600bps') ;
plot(el(1,:),squeeze(E_ebno_96(1,1,:)),'g') ;    
plot(el(1,:),squeeze(E_ebno_96(1,2,:)),'b') ;
plot(el(1,:),squeeze(E_ebno_96(1,3,:)),'r') ;
legend('400 km', '600 km','800 km') ;
xlabel('Elevation Angle [degrees]') ;
ylabel('Eb/No [dB]') ;
axis([ 0 90 -30 30 ]) ;
grid on

% Plot LM vs Elevation for both modulations
figure
hold on
title('Link Margin for BFSK') ;
plot(el(1,:),squeeze(E_LM_ebno_96_fsk(1,1,:)),'g') ;
plot(el(1,:),squeeze(E_LM_ebno_96_fsk(1,2,:)),'b') ;
plot(el(1,:),squeeze(E_LM_ebno_96_fsk(1,3,:)),'r') ;
legend('400 km', '600 km','800 km') ;
xlabel('Elevation Angle [degrees]') ;
ylabel('Link Margin (Eb/No) [dB]') ;
axis([ 0 90 -30 30 ]) ;
grid on

figure
hold on
title('Link Margin for BPSK') ;
plot(el(1,:),squeeze(E_LM_ebno_96_psk(1,1,:)),'g') ;
plot(el(1,:),squeeze(E_LM_ebno_96_psk(1,2,:)),'b') ;
plot(el(1,:),squeeze(E_LM_ebno_96_psk(1,3,:)),'r') ;
legend('400 km', '600 km','800 km') ;
xlabel('Elevation Angle [degrees]') ;
ylabel('Link Margin (Eb/No) [dB]') ;
axis([ 0 90 -30 30 ]) ;
grid on


% Plot Eb/No vs. Elevation and Eb/No Threshold BER=10e-5
figure
hold on
title('Eb/No received at 600 KM') ;
plot(el(1,:),squeeze(E_ebno_96(1,2,:)),'b') ;    %Eb/No, Rate=9600
plot(el(1,:),EbNo_thr_fsk,'g') ;
plot(el(1,:),EbNo_thr_psk,'m') ;
plot(el(1,:),EbNo_thr_cqpsk,'c') ;
xlabel('Elevation Angle [degrees]') ;
ylabel('Eb/No [dB]') ;
axis([ 0 90 -30 40 ]) ;
%legend('Eb/No Received R=9600bps', 'Eb/No Threshold FSK', 'Eb/No Threshold PSK' ,'Eb/No Threshold Coded 4-PSK' ) ;
grid on



