function[data sep]=read_stk_elev(fname)
% Function to read STK data files containing elevation. For now it only
% supports elevation . The usage is as follows :
%
% You set up a scenario with a satellite and a ground station facility in
% STK and you export the elevation as seen from the ground station to a
% .dat file . You have to export ONLY the elevation and not azimuth and
% range since read_stk_elev ( ) doesn't support that . If done correctly , all
% the lines in the .dat file should look like the following :
%
% Elevation (deg) 47 1.1.2012 03:36:15,000 2,27845993613939
%
% The first number is simple an index, the number of the sample.
% Subsequently follows the date and time , and finally the elevation angle
% in degrees .
%
% There is an issue with the format of the .dat file STK produces; they
% cannot be read by MATLAB. To fix that, open them in a text editor and
% save them with UTF-8 encoding before using read_stk_elev ( ) .
%
% Import the data to a MATLAB matrix called data by
%
% data=read_stk_elev(fname)
%
% where fname is a variable (of type string) containing the file name (or 
% URL) of the .dat-file exported from STK.
%
% data will be a N x 7 matrix where N is the number of samples in the
% .dat file from STK. The first dimension is the number of samples whereas
% the second dimension holds different information (fields) about the
% samples :
%
% 1-Year
% 2-Month
% 3-Day
% 4-Hour
% 5-Minute
% 6-Second
% 7-Elevation
%
% For example, data (23,7) yields the elevation of the 23rd sample.
%
% Note that between the passes , when the satellite is not empty , STK
% produces an empty (separator) sample . For these samples all the fields
% are set to 0 in the data matrix. In some cases , it will be useful for the
% user to know which samples are separators. Instead of testing for it, the
% user can get a list of separators, sep, in the following way :
%
% [data sep]=read_stk_elev(fname)
%
% Made by :
% Marianne Bakken <mariba@stud.ntnu.no >, MSc. thesis for NUTS 2012
% Sigvald Marholm <marholm@stud.ntnu.no >, MSc. thesis for NUTS 2012
%
% Copyright 2012 , NUTS ? NTNU Test Satellite
%



% A typical line may look like the following, followed by regexp blocks and
% name of variables they will be stored into .

% Elevation (deg) 47 1.1.2012 03:36:15,000 2,27845993613939
% ---%s---- -%s-- %d ---%s--- -----%s----- -------%s-------
% index date time elevation

% The two first blocks are just waste
% The %d can be used as index in a matrix
% The three last blocks are read as strings to be parsed later

pattern='%s %s %d %s %s %s' ;

fid=fopen(fname) ;
unparsed=textscan(fid,pattern) ;
fclose(fid) ;

% Read out the regexp blocks
index=unparsed(3) ;
date=unparsed(4) ;
time=unparsed(5) ;
elevation=unparsed (6) ;

% MATLAB stores them as a 1-element cell-array for some reason .
% Read out the one cell element .
index=index{1};
date=date{1};
time=time{1};
elevation=elevation{1};

N=size(index,1) ; % The output is a Nx7 matrix

% Reads out the data in a Nx7 matrix of the form data (index,parameter)
% where index is the same index as in the file , parameter is a value of
% the following :
%
% 1 - year
% 2 - month
% 3 - day
% 4 - hour
% 5 - minute
% 6 - second
% 7 - elevation
%
% So for example data (23,:) yields the time and elevation for sample 23 .

% Initialize data matrix and sep vector
data=zeros(N,7) ;
sep=[] ;

for i=index.'

  date_=date(i) ;
  time_=time(i) ;
  elevation_=elevation(i) ;

  % For some reason MATLAB stores this into 1-dimensional cell arrays as well
  date_=date_{1};
  time_=time_{1};
  elevation_=elevation_{1};

  if (isempty(elevation_))
    % STK Produces an empty row with just an index when the satellite
    % is not visible. Reg. exp. functions run into trouble if they try
    % to parse an empty string. The convention will be that the matrix
    % is left with zeros at these places. A vector sep is available as
    % an output for the user. This is an easy way for the user to keep
    % track of where these samples are .
    sep=[sep i] ;
  else

    % Replace delimiters with space
    %%%date_=regexprep(date_,'\.',' '); % '1.1.2012 ' => '1 1 2012 '
    
    date_=regexprep(date_,'/',' '); % '1.1.2012 ' => '1 1 2012 '
     
    time_=regexprep(time_, ':',' ' ) ; % '03:36:15,000' => '03 36...15000'
   
    % Replace decimal,with .
    time_=regexprep(time_,' , ' , ' . ' ) ; % '03 36 15 ,000 ' => '03 36 ...15 .000 '
    %%% elevation_=regexprep(elevation_, ' , ' , ' . ' ) ; % '2,27845993613939'=> ...'2.27845993613939'

    % Read out 'day month year' from date_
    temp=textscan(date_, '%d %d %d ' , 'CollectOutput',1) ;
    temp=temp{1};

    data(i,1)=temp(3) ; % year
    data(i,2)=temp(2) ; % month
    data(i,3)=temp(1) ; % day

    % Read out 'hour minute second' from time_
    temp=textscan(time_,'%f %f %f ' , 'CollectOutput',1) ;
    temp=temp{1};

    data(i,4)=temp(1); % hour
    data(i,5)=temp(2); % minute
    data(i,6)=temp(3); % second

    % Read out 'elevation' from elevation_
    temp=textscan(elevation_, '%f ' , 'CollectOutput',1) ;
    temp=temp{1};

    data(i,7)=temp(1); %elevation

end

end

end
