function intervals=threshold_stk_elev(data,threshold)
% Given a set of elevation data from STK simulations and read with
% read_stk_data( ), this function determines the time intervals when the
% elevation is higher than a certain threshold, i.e. 10 degrees.
%
% Usage :
%
% intervals=threshold_stk_elev(data,threshold)
%
% where data is the output data matrix from read_stk_data( ) and threshold
% is the elevation threshold.
%
% The output is a matrix where each interval is represented by one row of
% 12 elements. The first 6 elements of the row are the year, month, day,
% hour, minute and second of the start of the period. The 6 last elements
% represent the end of the period in the same manner.
%
% Example output :
%
% intervals =
%
% 1 1 2012 13 43 23 1 1 2012 13 48 43
% 1 1 2012 15 32 11 1 1 2012 15 42 12
%
% This example shows that the data contains two intervals where the
% elevation is higher than the user specified threshold elevation. The
% second interval, for example, starts on 1.1.2012 15:32:11 and ends
% 15:42:12 the same day.

intervals=[];

% The algorithm works by iterating through all samples and detect when the
% elevation increase above, or decrease below, the given threshold . When
% that happens, it writes to the intervals array. It remembers whether or
% not the elevation is above the threshold by a flag called above.

above=0; % Nominally not in an interval
N=size(data,1) ;
data=stk_remove_spurious(data) ;
%data=read_stk_elev(data);

for i=1:N

 if((data(i,7)>=threshold)&&~above)

 if(~all(data(i,1:6)==[0 0 0 0 0 0])) % Ignore empty samples

  % Sometimes STK produces spurious output (unbelievably enough).
  % When the elevation passes below 1 degree it starts typing them in
  % exponential form, i.e. 7,442341 e-001 except that sometimes it
  % doesn't get the exponent right. Instead the exponent becomes 0,
  % and the reisa single sample before the satellite goes down where
  % the elevation is supposedly 7 degrees. This function handles that
  % by checking that the next sample is also above the threshold .

  if (i+1)<=N % Check that there is a next sample and ignore if not .

    if (data(i+1,7)>=threshold)
        
      % An interval starts at this sample. Add a new row in
      % intervals and add the time in the six first columns .

      intervals(end+1,1:6)=data(i,1:6) ;
      above=1 ;
    end
  end
 end

elseif((data(i,7)<threshold)&&above)

  % An interval stops at this sample. Add the time to the six last
  % columns in the last row.
  intervals(end,7:12)=data(i,1:6);
  above=0;

  elseif(all(data(i,1:6)==[0 0 0 0 0 0 ])&&above)

    % if threshold = 0 the above mechanism to detect an end of the
    % interval doesn't work since STK doesn't save samples with
    % negative elevation, and hence all samples will have value s >=0.
    % Instead , STK saves a separator sample between every pass where
    % all fields (year,etc. ) are simply zero. When such an sample
    % occur, and the above flag is still set, you know that the
    % PREVIOUS sample was the last one .

    intervals(end,7:12)=data(i-1,1:6);
    above=0 ;

  end

end
